/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef DPC_IMPL_TYPE_SLPOINT_H
#define DPC_IMPL_TYPE_SLPOINT_H

#include <cfloat>
#include <cmath>
#include "impl_type_float.h"

namespace dpc {
struct SLPoint {
    ::Float s{ 0.0F };
    ::Float ds{ 0.0F };
    ::Float dds{ 0.0F };
    ::Float dds_valid_time{ 0.0F };
    ::Float l{ 0.0F };
    ::Float dl{ 0.0F };
    ::Float dl_valid_time{ 0.0F };
    ::Float ddl{ 0.0F };
    ::Float ddl_valid_time{ 0.0F };

    static bool IsPlane()
    {
        return true;
    }


    using IsEnumerableTag = void;
    template<typename F>
    void enumerate(F& fun)
    {
        fun(s);
        fun(ds);
        fun(dds);
        fun(dds_valid_time);
        fun(l);
        fun(dl);
        fun(dl_valid_time);
        fun(ddl);
        fun(ddl_valid_time);
    }

    template<typename F>
    void enumerate(F& fun) const
    {
        fun(s);
        fun(ds);
        fun(dds);
        fun(dds_valid_time);
        fun(l);
        fun(dl);
        fun(dl_valid_time);
        fun(ddl);
        fun(ddl_valid_time);
    }

    template<typename F>
    void enumerate_internal(F& fun)
    {
        fun("s", s);
        fun("ds", ds);
        fun("dds", dds);
        fun("dds_valid_time", dds_valid_time);
        fun("l", l);
        fun("dl", dl);
        fun("dl_valid_time", dl_valid_time);
        fun("ddl", ddl);
        fun("ddl_valid_time", ddl_valid_time);
    }

    template<typename F>
    void enumerate_internal(F& fun) const
    {
        fun("s", s);
        fun("ds", ds);
        fun("dds", dds);
        fun("dds_valid_time", dds_valid_time);
        fun("l", l);
        fun("dl", dl);
        fun("dl_valid_time", dl_valid_time);
        fun("ddl", ddl);
        fun("ddl_valid_time", ddl_valid_time);
    }

    friend bool operator==(const ::dpc::SLPoint& lhs, const ::dpc::SLPoint& rhs) noexcept
    {
        return (fabs(static_cast<double>(lhs.s - rhs.s)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.ds - rhs.ds)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.dds - rhs.dds)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.dds_valid_time - rhs.dds_valid_time)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.l - rhs.l)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.dl - rhs.dl)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.dl_valid_time - rhs.dl_valid_time)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.ddl - rhs.ddl)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.ddl_valid_time - rhs.ddl_valid_time)) < DBL_EPSILON);
    }
};
} // namespace dpc


#endif // DPC_IMPL_TYPE_SLPOINT_H
